#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ConfigParser import ConfigParser

from requests_oauthlib import OAuth1Session

# FIXME: thomi where would I expect to find?
NFSS_CONFIG_PATH = "FIXME-fake-nfss-config-path"


class NfssClientError(Exception):
    """Non-functional non-functional stats service."""


def parse_nfss_config(nfss_config_path):
    """Return a ConfigParser config from NFSS config filepath.

    NOTE that we only return the 'nfss' section of the config.

    :param nfss_config_path: FIXME
    """
    config = ConfigParser()
    config.read(nfss_config_path)
    # are all of our ducks in a row?
    for config_item in (
            'client_access_key',
            'resource_owner_key',
            'resource_owner_secret',
            'backend'):
        # (we'll also raise KeyError on missing 'nfss' section)
        if config_item not in config['nfss']:
            raise KeyError(config_item)
    return config['nfss']


class NfssClient(object):

    def __init__(self, nfss_config_path=None):
        if nfss_config_path is None:
            self.nfss_config_path = NFSS_CONFIG_PATH
        try:
            self.config = parse_nfss_config(nfss_config_path)
        except KeyError as e:
            raise NfssClientError('Failed to parse NFSS config: '
                                  '%s' % e.message)
        self.session = OAuth1Session(
            self.config['client_access_key'],
            resource_owner_key=self.config['resource_owner_key'],
            resource_owner_secret=self.config['resource_owner_secret'],
        )

    def post_data(self, json, project_name, test_name):
        """POST JSON result data to NFSS at a given project and test."""
        url = '/'.join((self.config['backend'], project_name, test_name))
        r = self.session.post(url, dict(data=json))
        if r.status_code != 200:
            raise NfssClientError("Failed to POST to NFSS: %s" % r.content)
