#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci_utils import amqp_utils, amqp_worker
from coverageextractor.extraction import Extraction, ExtractionFailed


class CoverageExtractorWorker(amqp_worker.AMQPWorker):

    def __init__(self):
        super(CoverageExtractorWorker, self).__init__('coverageextractor')

    def handle_request(self, params, logger):
        """Extract coverage numbers from coverage.xml and post to NFSS.

        :param params: dictionary of params from incoming message
        :param logger: logging.Logger for reporting
        """
        ticket_id = params['ticket_id']
        data_store = self._create_data_store(ticket_id)
        extraction = Extraction(
            ticket_id,
            logger,
            data_store,
        )
        try:
            project, version = extraction.try_get_project_and_version(params)
            coverage_xml_path = extraction.try_get_coverage_xml_path(params)
            coverage_xml = extraction.try_get_coverage_xml(coverage_xml_path)
            line_data, branch_data = extraction.try_extract_coverage_data(
                coverage_xml
            )
            data = extraction.try_compose_nfss_json(
                project,
                version,
                line_data,
                branch_data,
            )
            extraction.try_post_data_to_nfss(data, project)
        except ExtractionFailed as e:
            return e.return_tuple
        return extraction.report_succeeded(ticket_id)


if __name__ == '__main__':
    CoverageExtractorWorker().main(amqp_utils.COVERAGE_RETRIEVER_QUEUE)
