# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals

from ci_utils.tastypie.test import TastypieTestCase
from django.contrib.auth.models import User


class UserInfoTest(TastypieTestCase):

    def setUp(self):
        super(UserInfoTest, self).setUp('/api/v1')

    def test_anonymous_info(self):
        # Anonymous requests to /userinfo/ returns an object with
        # 'authenticated' set as False. The other fields are bogus/empty.
        user_info = self.getResource('userinfo/')
        self.assertEqual({
            'authenticated': False,
            'username': '',
            'email': '',
            'fullname': '',
            'staff': False,
            'superuser': False,
        }, user_info)

    def test_authenticated_info(self):
        # Authenticated requests to /userinfo/ returns the user information.
        password = '1234'
        user = User.objects.create_user(
            username='test', first_name='First', last_name='Last',
            password=password, email='t@t.com')
        self.client.client.login(username=user.username, password=password)

        user_info = self.getResource('userinfo/')
        self.assertEqual({
            'authenticated': True,
            'username': 'test',
            'email': 't@t.com',
            'fullname': 'First Last',
            'staff': False,
            'superuser': False,
        }, user_info)
