"use strict";

// Make jslint happy
var YUI;

YUI.add('webui', function (Y) {
    var json_error_msg = (
        "The ticket system returned data that could not be understood. " +
        "Please try refreshing, or contact support if this problem " +
        "persists."),
        refresh_msg = "Refreshing data...",
        default_refresh = 60,
        static_prefix = "/static/common/";
    Y.webui = {
        url_prefix: "/api/v1/",
        refresh_interval: default_refresh * 1000,
        refresh_timer: null,
        done_loading: false,
        log_prefixes: Y.Array(
            ['package_build', 'image_build', 'image_test',
             'package_publish']),
        log_step_map: {
            "Image building": "image_build",
            "Image testing": "image_test",
            "Package building": "package_build",
            "Package publishing": "package_publish"
        },
        logs: {},
        tempurls: {},
        get_log_name: function (step_name) {
            var log_name = Y.webui.log_step_map[step_name];

            if (!log_name) {
                log_name = "misc";
            }

            return log_name;
        },
        add_log: function (log_data) {
            var key = log_data.name.split('.')[0];

            if (Y.Array.indexOf(Y.webui.log_prefixes, key) === -1) {
                key = "misc";
            }

            if (!Y.webui.logs[key]) {
                Y.webui.logs[key] = [];
            }

            Y.webui.logs[key].push(log_data);
        },

        /**
         * Setup page auto-refresh.
         *
         * Immediately runs the callback when auto-refresh is set up
         * (page is loaded) then use Y.later() for periodically calling
         * the 'callback' at the `Y.webui.refresh_interval` rate.
         *
         * @method setup_auto_refresh
         */
        setup_auto_refresh: function (callback) {
            callback();
            Y.webui.refresh_timer = Y.later(
                Y.webui.refresh_interval, Y, callback, '', true);
        },

        get_workflow_data: function (api_url) {
            var wf_data = {},
                wf_str = {},
                wf_num = {},
                i,
                item;

            Y.io(Y.webui.url_prefix + api_url, {
                sync: true,
                on: {
                    success: function (tx, r) {
                        try {
                            wf_data = Y.JSON.parse(r.responseText).objects;
                        } catch (e) {
                            Y.webui.main_table.showMessage(json_error_msg);
                            return;
                        }
                    }
                }
            });

            for (i = 0; i < wf_data.length; i += 1) {
                item = wf_data[i];

                wf_str[item.name] = item.value;
                wf_num[item.value] = item.name;
            }

            return {
                wf_data_list: wf_data,
                wf_data_list_str: wf_str,
                wf_data_list_num: wf_num
            };
        },
        get_workflow_statuses: function () {
            var result, api_url = "ticket_workflow_step_status/";

            result = Y.webui.get_workflow_data(api_url);

            Y.webui.workflow_status_list = result.wf_data_list;
            Y.webui.workflow_status_list_str = result.wf_data_list_str;
            Y.webui.workflow_status_list_num = result.wf_data_list_num;
        },
        get_workflow_steps: function () {
            var result, api_url = "ticket_workflow_step/";

            result = Y.webui.get_workflow_data(api_url);

            Y.webui.workflow_step_list = result.wf_data_list;
            Y.webui.workflow_step_list_str = result.wf_data_list_str;
            Y.webui.workflow_step_list_num = result.wf_data_list_num;
        },
        get_workflow_step_from_status: function (wf_status_str) {
            var status_num, step_num, step_name, step_factor;

            status_num = Y.webui.workflow_status_list_str[wf_status_str];

            step_factor = Math.floor(status_num / 100);
            step_num = step_factor * 100;

            step_name = Y.webui.workflow_step_list_num[step_num];

            return step_name;
        },
        get_ticket: function (ticket_id, elem) {

            if (Y.webui.main_table) {
                Y.webui.main_table.showMessage("loading data...");
            }
            // Load ticket information from TS.
            Y.io(Y.webui.url_prefix + "fullticket/" + ticket_id + "/", {
                sync: true,
                on: {
                    success: function (tx, r) {
                        var data, new_div;

                        try {
                            data = Y.JSON.parse(r.responseText);
                        } catch (e) {
                            Y.webui.main_table.showMessage(json_error_msg);
                            return;
                        }

                        new_div = Y.webui.ticket_detail_element(data);
                        if (elem) {
                            elem.setHTML(new_div);
                        }
                        new_div = null;

                    },
                    failure: function (tx, r, e) {
                        console.log("failure loading fullticket (" +
                                    ticket_id + "): ", tx, r, e);
                    }
                }
            });
        },
        add_subticket: function (data) {
            var main_tr = Y.Node.create("<tr>"),
                key,
                value,
                tr;

            main_tr.appendChild(Y.Node.create('<th>subticket</th>'));
            for (key in data) {
                if (data.hasOwnProperty(key)) {
                    value = data[key];
                    tr = main_tr.appendChild(Y.Node.create('<tr>'));
                    tr.addClass('subticket');
                    tr.appendChild(Y.Node.create('<th>' + key + '</th>'));
                    tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
                }
            }

            main_tr.addClass('subticket');
            return main_tr;
        },
        make_subticket: function (data) {
            var div = Y.Node.create("<div class='subticket'></div>"),
                table = div.appendChild(Y.Node.create("<table>")),
                hidden_fields = Y.Array([
                    'id',
                    'resource_uri',
                    'current_workflow_step',
                    'status',
                    'source_package_upload',
                ]),
                key,
                value,
                tmp_th,
                tr,
                count,
                suffix,
                span;

            for (key in data) {
                if (data.hasOwnProperty(key)) {
                    value = data[key];

                    // Skip hidden fields
                    if (Y.Array.indexOf(hidden_fields, key) === -1) {

                        suffix = "";
                        if (key === 'artifact') {
                            span = Y.webui.add_artifacts(data.artifact);
                            value = span.getHTML();
                            count = value.match(/<a/g);
                            if (count && count.length > 1) {
                                suffix = "s";
                            }
                        }

                        if (key === 'source_package_upload') {
                            span = Y.webui.sp_upload(data.source_package_upload);
                            value = span.getHTML();
                        }

                        key = key.replace("_", " ");
                        if (value === null) {
                            value = "";
                        }

                        // subticket
                        tr = table.appendChild(Y.Node.create('<tr>'));
                        tmp_th = ('<th>' + key.charAt(0).toUpperCase() +
                                  key.slice(1) + suffix + ': </th>');
                        tr.appendChild(Y.Node.create(tmp_th));
                        if (key === 'conflicts') {
                            // Display 'conflicts' as 'ticket' links.
                            var td = Y.Node.create('<td>')
                                .addClass('conflicts');
                            for (var i in value) {
                                var icon = Y.Node.create('<img>')
                                    .set('src', '/static/common/cross.svg');
                                td.appendChild(icon);
                                var href = value[i].replace('/api/v1', '');
                                var text = href.replace(
                                    '/ticket/', '').replace('/', '');
                                var link = Y.Node.create('<a>')
                                    .set('href', href)
                                    .set('text', text);
                                td.appendChild(link);
                                td.appendChild('</br>');
                            }
                            tr.appendChild(td);
                        } else {
                            // Display other fields as plain-text.
                            tr.appendChild(
                                Y.Node.create('<td>' + value + '</td>'));
                        }
                    }
                }
            }

            return div;
        },
        basename: function (path) {
            var result = path;

            if (path) {
                result = path.slice(path.lastIndexOf('/') + 1, path.length);
            }

            return result;
        },
        add_artifacts: function (data) {
            var span = Y.Node.create('<span></span>'),
                i;

            for (i = 0; i < data.length; i += 1) {
                Y.webui.add_log(data[i]);
                span.appendChild(Y.webui.artifact_link(data[i]));
                span.appendChild(Y.Node.create("<br/>"));
            }

            return span;
        },
        artifact_link: function (data, new_name) {
            var name = Y.webui.basename(data.name),
                link = Y.Node.create('<a/>');

            // Lookup a corresponding tempurl.
            var tempurl = Y.webui.tempurls[name];

            if (new_name) {
                name = new_name;
            }
            link.set('text', name);

            // Link the artifact to the tempurl or fallback to the original
            // TS reference.
            if (tempurl === undefined) {
                console.log('No tempurl for:', data.name);
                link.set('href', data.reference);
            } else {
                link.set('href', tempurl);
            }

            return link;
        },
        /**
         * Build a link to ppa reference (ppa:<user>/[<distro>/]<name>)
         *
         * Returns a new <a> element for valid reference, otherwise a <span>.
         *
         * @method ppa_link
         */
        ppa_link: function(reference) {
            var matches = reference.match(
                /ppa:([^\/]+)\/([^\/]+)(\/([^\/]+)|)/);
            if (!matches) {
                return Y.Node.create('<span>')
                    .set('text', reference);
            }
            var username = matches[1];
            var distribution = matches[2];
            var name = matches[3];
            if (name) {
                name = name.replace('/', '');
            } else {
                name = distribution;
                distribution = 'ubuntu';
            }
            var href = (
                'https://launchpad.net/~' + username +
                '/+archive/' + distribution + '/' + name);

            return Y.Node.create('<a>')
                .set('href', href)
                .set('text', reference)
                .addClass('ppa-link');
        },
        sp_upload: function (data) {
            var span = Y.Node.create("<span></span>");

            if (data.sourcepackage) {
                span.setHTML(data.sourcepackage.name + " " + data.version);
            }

            return span;
        },
        ticket_detail_element: function (data) {
            var MAX_DESCRIPTION = 200,
                count,
                current_step,
                description = "",
                description_p,
                div,
                fail_str,
                found_step,
                header_description,
                header_wrapper,
                i,
                key,
                log_name,
                not_yet_step,
                short_status,
                skipped_step,
                span,
                status_image,
                step_status,
                subt,
                subticket,
                subticket_count,
                subticket_wrapper,
                suffix,
                table,
                td,
                td1,
                ticket_div,
                ticket_wrapper,
                tmp_th,
                tr,
                value,
                wf_data,
                wf_table,
                workflow_inner_div,
                workflow_wrapper;

            // Reset logs lists
            Y.webui.logs = {};

            if (data.title) {
                div = Y.Node.create("<div id='table_wrapper'/>");

                // These determine the ordering of the divs on the page.
                header_wrapper = div.appendChild(Y.Node.create('<div class="header_wrapper"></div>'));
                workflow_wrapper = div.appendChild(Y.Node.create('<div class="workflow_wrapper"></div>'));
                ticket_wrapper = div.appendChild(Y.Node.create('<div class="ticket_wrapper"></div>'));
                subticket_wrapper = div.appendChild(Y.Node.create('<div class="subticket_wrapper"></div>'));

                var title = Y.Node.create('<h2>')
                    .set('text', data.title);
                header_wrapper.appendChild(title);
                var uuid = Y.Node.create('<h3>');
                uuid.appendChild(
                    Y.Node.create('<small>').set('text', data.uuid));
                header_wrapper.appendChild(uuid);

                if (data.description) {
                    if (data.description.length > MAX_DESCRIPTION) {
                        description = data.description.slice(0, MAX_DESCRIPTION) + "...";
                    } else {
                        description = data.description;
                    }
                }

                header_description = header_wrapper.appendChild(
                    Y.Node.create('<div class="header_description"></div>')
                );
                description_p = header_description.appendChild(Y.Node.create('<p></p>'));
                description_p.appendChild(Y.Node.create('<pre>' + description + '</pre>'));
                workflow_wrapper.appendChild("<h3>Landing Progress</h3>");

                workflow_inner_div = workflow_wrapper.appendChild(Y.Node.create("<div class='workflow'></div>"));

                wf_table = workflow_inner_div.appendChild(Y.Node.create('<table class="workflow_steps"></table>'));
                fail_str = "Failed";

                wf_table.appendChild(Y.Node.create('<tr><th>Step</th>' +
                    '<th class="workflow_status">Status</th>' +
                    '</tr></table>')
                    );

                wf_data = Y.webui.workflow_step_list;

                short_status = data.status.slice(data.status.length - fail_str.length, data.status.length);

                found_step = false;
                if (data.current_workflow_step !== "Failed") {
                    current_step = data.current_workflow_step;
                } else {
                    current_step = Y.webui.get_workflow_step_from_status(data.status);
                }
                for (i = 0; i < wf_data.length; i += 1) {
                    step_status = 'Completed';
                    status_image = null;
                    skipped_step = false;

                    if (!found_step) {
                        step_status = 'Completed';
                        status_image = static_prefix + 'check.svg';
                    }

                    if (wf_data[i].name === "Completed" || wf_data[i].name === "Failed" ||
                            wf_data[i].name === "New") {
                        skipped_step = true;
                    }

                    if (!found_step && (current_step === wf_data[i].name)) {
                        found_step = true;
                        if (data.status !== "Completed" && short_status !== "Failed") {
                            if(data.status === 'Paused') {
                                step_status = "Needs Attention";
                                status_image = static_prefix + 'paused.svg';
                            } else {
                                step_status = "In Progress";
                                status_image = static_prefix + 'spinner3.gif';
                            }
                        } else {
                            step_status = data.status;
                            if (short_status === "Failed") {
                                status_image = static_prefix + "cross.svg";
                            } else {
                                status_image = static_prefix + "check.svg";
                            }
                        }
                    }

                    if (not_yet_step) {
                        step_status = "Failed";
                    }

                    if (data.current_workflow_step === "Failed" && wf_data[i].name === "Failed") {
                        step_status = "Failed";
                        status_image = static_prefix + "cross.svg";
                    }

                    if (!skipped_step) {
                        tr = Y.Node.create('<tr></tr>');
                        wf_table.appendChild(tr);
                        td1 = Y.Node.create('<td class="workflow_col1">' + wf_data[i].name + ': </td>');
                        if (found_step && !not_yet_step) {
                            td1.addClass('current_step');
                        }
                        tr.appendChild(td1);
                        td = tr.appendChild('<td class="workflow_status_td"></td>');
                        if (status_image) {
                            var elem = '<img src="' + status_image + '" title="' + step_status + '">';
                            if(data.status === 'Paused') {
                                elem = '<a href="/ticket/' + data.uuid + '/review">' + elem + '</a>';
                            }
                            td.appendChild(elem);
                        }

                        log_name = Y.webui.get_log_name(wf_data[i].name);

                        tr = wf_table.appendChild(Y.Node.create('<tr></tr>'));
                        td = Y.Node.create('<td colspan="2" class="artifact_td"></td>');
                        if (log_name !== "misc") {
                            span = Y.Node.create('<span id="' + log_name + '"></span>');
                            td.appendChild(span);
                        }

                        tr.appendChild(td);
                    }

                    if (found_step) {
                        not_yet_step = true;
                    }
                }

                if (short_status === "Failed") {
                    wf_table.appendChild(Y.Node.create("<tr></tr>")).appendChild(
                        Y.Node.create("<td colspan='2' class='red'>" + short_status + "</td>")
                    );
                }

                if (current_step === "Completed") {
                    wf_table.appendChild(Y.Node.create("<tr></tr>")).appendChild(
                        Y.Node.create("<td colspan='2' class='green'>Success</td>")
                    );
                }
            }

            // Create a section for presenting ticket details.
            ticket_wrapper.appendChild('<h3>Ticket Details</h3>');
            ticket_div = ticket_wrapper.appendChild(
                Y.Node.create("<div class='ticket'></div>"));

            // Build and render the ticket details datatable.
            var dt = Y.webui.get_ticket_details_datatable(data);
            dt.render(ticket_div);

            // Add an extra row with a link for accessing Lander 'live' logs
            // if the ticket is still in progress.
            if (current_step !== "Completed" && short_status !== "Failed") {
                var watch_log = Y.Lang.sub(
                    '<a href="livelog?lander_unit={lander_unit}">' +
                    'Watch Build</a>', data)
                dt.addRow({'value': watch_log});
            }

            if (data.subticket) {
                subticket_count = data.subticket.length;

                subticket_wrapper.appendChild("<h3>Uploads</h3>");
                if (subticket_count > 0) {
                    for (i = 0; i < subticket_count; i += 1) {
                        subticket = data.subticket[i];

                        subticket_wrapper.appendChild("<h4>" + data.subticket[i].source_package_upload.sourcepackage.name + " " + data.subticket[i].source_package_upload.version + "</h4><div id='status'><strong>Landing Progress:</strong> " + data.subticket[i].current_workflow_step + " <strong>Status:</strong> " + data.subticket[i].status + "</div>");

                        subt = Y.webui.make_subticket(subticket);

                        subticket_wrapper.appendChild(subt);
                    }
                }
            }

            Y.webui.populate_log_tds(wf_table);
            wf_table = null;

            return div;
        },

        /**
         * Build a datatable for presenting details of the given Ticket.
         *
         * Returns a Y.Datatable loaded with ticket details.
         *
         * @method get_ticket_details_datatable
         */
        get_ticket_details_datatable: function(ticket) {

            var dataset = [];
            var fields = [
                'artifact',
                'creator', 'landers', 'created', 'updated', 'private',
                'bug_id', 'sync_request',
                'series', 'base_image', 'master_ppa', 'build_ppa',
                'job_url', 'test_notes', 'comments',
            ];

            if (ticket.citrain_overlay) {
                ticket = Y.merge(ticket, ticket.citrain_overlay);
            }

            Y.Array.each(fields, function (key) {
                if (!ticket.hasOwnProperty(key)) return;
                dataset.push({'key': key, 'value': ticket[key]});
            });

            var format_key = function (o) {
                if (!o.data.key) return '';
                var text = o.data.key.replace('_', ' ');
                return text.charAt(0).toUpperCase() + text.slice(1) + ':';
            }

            var format_value = function (o) {
                var value;
                switch (o.data.key) {
                    case 'artifact':
                        value = Y.webui.add_artifacts(o.data.value).getHTML();
                        break;
                    case 'bug_id':
                        if (!o.data.value) {
                           break;
                        }
                        value = Y.Lang.sub(
                            '<a class="bug-link" ' +
                            'href="https://launchpad.net/bugs/{bug_id}">' +
                            '#{bug_id}</a>',
                            {bug_id: o.data.value});
                        break;
                    case 'base_image':
                        var label = o.data.value.slice(
                            o.data.value.lastIndexOf('/') + 1);
                        value = Y.Lang.sub(
                            '<a class="image-link" href="{href}">{label}</a>',
                            {href: o.data.value, label: label});
                        break;
                    case 'comments':
                    case 'test_notes':
                        value = Y.Lang.sub(
                            '<textarea readonly style="width: 80%;">' +
                            '{content}</textarea>',
                            {content: o.data.value});
                        break;
                    case 'job_url':
                        var label = o.data.value;
                        value = Y.Lang.sub(
                            '<a class="job-link" href="{href}">{label}</a>',
                            {href: o.data.value, label: label});
                        break;
                    case 'build_ppa':
                    case 'master_ppa':
                        var n = Y.Node.create('<div>');
                        n.appendChild(Y.webui.ppa_link(o.data.value));
                        value = n.getHTML();
                        break;
                    default:
                        value = o.data.value;
                }
                return value;
            }

            return new Y.DataTable({
                columns: [
                    {key: 'key', label: 'Key', 'width': '150%',
                     allowHTML: true, formatter: format_key,
                     className: 'dt-details-key'},
                    {key: 'value', label: 'Value',
                     allowHTML: true, formatter: format_value,
                     className: 'dt-details-value'}
                ],
                data: dataset
            });
        },

        populate_log_tds: function (wf_table_log) {
            var key,
                log_td,
                logs,
                i,
                td_id;

            for (key in Y.webui.logs) {
                if (Y.webui.logs.hasOwnProperty(key)) {
                    td_id = '#' + key;
                    log_td = wf_table_log.one(td_id);

                    if (log_td) {
                        logs = Y.webui.logs[key];

                        for (i = 0; i < logs.length; i += 1) {
                            if (log_td) {
                                log_td.appendChild(
                                    Y.webui.artifact_link(
                                        logs[i],
                                        logs[i].name.slice(logs[i].name.indexOf('.') + 1)
                                    )
                                );
                                log_td.appendChild('<br/>');
                            }
                        }
                    }
                }
            }
        },

        /**
         * Load a map of tempurls for the context artifacts from GK.
         *
         * Reset Y.webui.tempurls.
         *
         * @method load_tempurls
         */
        load_tempurls: function(ticket_id_or_uuid) {
            Y.webui.tempurls = {};
            Y.io("/gk/api/v1/ticket/" + ticket_id_or_uuid, {
                sync: true,
                on: {
                    success: function (tx, r) {
                        Y.webui.tempurls = Y.JSON.parse(r.responseText);
                    },
                    failure: function (tx, r, e) {
                        console.log("Failure loading tempurls (" +
                                    ticket_id_or_uuid + "): ", tx, r, e);
                    }
                }
            });
        },

        /**
         * Returns the ticket ID based on the window URL.
         *
         * Checks the known ticket page path (/ticket/<id>), then match
         * a valid integer ID.
         *
         * @method get_ticket_id
         */
        get_ticket_id: function () {
            var path = Y.config.win.location.pathname;

            // Ensure the URL is what we expect for the ticket page.
            var uuid_pattern = '[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}'
            var url_regexp = new RegExp(
                '\/ticket\/(\\d+|' + uuid_pattern + ')\/', 'g')
            if (!path.match(url_regexp)) {
                return null;
            }

            // Isolate given term (ID or UUID);
            var ticket_id_or_uuid = path.replace('/ticket/', '');
            ticket_id_or_uuid = ticket_id_or_uuid.replace('/', '');

            // Try to load tempurls. It will fail if the given term does
            // not match the one used for creating the swift container.
            Y.webui.load_tempurls(ticket_id_or_uuid);
            if (Object.keys(Y.webui.tempurls).length > 0 ) {
                return ticket_id_or_uuid;
            }

            // Let's approach TS and use another alternative.
            Y.io("/api/v1/ticket/" + ticket_id_or_uuid + '/', {
                sync: true,
                on: {
                    success: function (tx, r) {
                        var t = Y.JSON.parse(r.responseText);
                        if (ticket_id_or_uuid.match(
                                new RegExp(uuid_pattern, 'g'))) {
                            ticket_id_or_uuid = t['id'];
                        } else {
                            ticket_id_or_uuid = t['uuid'];
                        }
                    },
                    failure: function (tx, r, e) {
                        console.log(
                            "Failure finding ticket alternative key (" +
                            ticket_id_or_uuid + "): ", tx, r, e);
                    }
                }
            });

            // Now we can properly load tempurls and continue safely.
            Y.webui.load_tempurls(ticket_id_or_uuid);
            return ticket_id_or_uuid;
        },

        /**
         * Updates ticket information.
         *
         * @method update_ticket
         */
        update_ticket: function () {
            var ticket_id = Y.webui.get_ticket_id();
            if (ticket_id) {
                Y.webui.get_ticket(ticket_id, Y.one('#container'));
            }
        },

        /**
         * Setup a DataTable and a remote DataSource for listing tickets.
         *
         * 'type' can be one of 'open', 'complete', 'failed', otherwise
         * all tickets will be listed.
         *
         * @method setup_ticket_datatable
         */
        setup_ticket_datatable: function (container, type) {
            var itemsPerPage = 10;

            var source = Y.webui.url_prefix + 'ticket/';

            // Adjust datasource query arguments according to the
            // ticket list 'type'.
            var query = '?order_by=-id&page={page}&limit={itemsPerPage}&';
            var initial = '?order_by=-id&limit=' + itemsPerPage + '&';
            var clause = '';
            if (type === 'open') {
                // Tickets QUEUED, PAUSED, RESUMED, INPROGRESS.
                clause = 'status__lte=100';
                query += clause;
                initial += clause
            } else if (type === 'complete') {
                // Tickets COMPLETED.
                clause = 'status=200';
                query += clause;
                initial += clause
            } else if (type === "failed") {
                // Tickets FAILED.
                clause = 'status=300';
                query += clause;
                initial += clause
            }

            // Paginator container.
            var paginator = Y.Node.create('<div>')
                .set('id', 'paginator')
            Y.one('#main-content').appendChild(paginator);

            // Row formatters ...
            var link_it = function (o) {
                return '<a class="row_link" href="/ticket/' +
                    o.data.uuid + '/">' + o.value + '</a>';
            };
            var format_id = function (o) {
                o.rowClass = "data-row-" + o.data.uuid;
                return link_it(o);
            };

            var dt = new Y.DataTable({
                columnset: [
                    {key: 'title', label: 'Title',
                     formatter: format_id, allowHTML: true},
                    {key: 'current_workflow_step', label: 'Landing Progress',
                     formatter: link_it, allowHTML: true},
                    {key: 'status', label: 'Status',
                     formatter: link_it, allowHTML: true}
                ],
                paginator: new Y.PaginatorView({
                    model: new Y.PaginatorModel({
                        itemsPerPage: itemsPerPage,
                    }),
                    maxPageLinks: 2,
                    container: '#paginator',
                }),
                resizePaginator: true,
                paginationSource: 'server',
                requestStringTemplate: query,
                strings: {
                    'emptyMessage': 'No results available.'
                }
            });

            var ds = new Y.DataSource.IO({
                source: source,
            });

            ds.on('response', function(e) {
                if (e.data.status !== 200) {
                    console.log('Datasource loading failure!', e);
                }
            });

            ds.plug(Y.Plugin.DataSourceJSONSchema, {
                schema: {
                    resultListLocator: 'objects',
                    resultFields: [
                        'title', 'current_workflow_step', 'status', 'uuid'
                    ],
                    metaFields: {
                        totalItems: 'meta.total_count'
                    }
                }
            });

            dt.plug(Y.Plugin.DataTableDataSource, {
                datasource: ds,
                initialRequest: initial
            });

            // Setup a periodic refreshing tasks.
            Y.later(Y.webui.refresh_interval, Y, function () {
               dt.refreshPaginator();
            }, '', true);

            // Finally render the ticket list.
            dt.render(container);

            // Globalize datatable, for UI notifications when loading
            // workflow steps and statuses. Although, they are most likely
            // to be overridden by datasource updates.
            Y.webui.main_table = dt;
        }
    };

    Y.webui.get_workflow_steps();
    Y.webui.get_workflow_statuses();
}, '0.0.1', {requires: ['io', 'node', 'json-parse', 'datatable']});
