"use strict";

// Make jslint happy
var YUI;

YUI.add('webuiforms', function (Y) {

    /**
     * Overriding Y.SelectField for properly rendering '<label>'s.
     */
    Y.WebUISelectField = Y.Base.create(
        'select-field', Y.SelectField, [Y.WidgetParent, Y.WidgetChild], {

        _renderLabelNode : function () {
            var contentBox = this.get('contentBox'),
                titleNode = Y.Node.create('<label></label>');
            titleNode.set('innerHTML', this.get('label'));
            contentBox.appendChild(titleNode);
            this._labelNode = titleNode;
        }
    });

    Y.webuiforms = {

        /**
         * Hardcoded array of series choices, must be kept in sync with
         * the supported options in Ticket-System.
         */
        seriesChoices: [
            {label : 'Utopic - 14.10', value : 'utopic'},
            {label : 'Trusty - 14.04', value : 'trusty'},
            {label : 'Precise - 12.04', value : 'precise'}
        ],

        /**
         * Setup a form for creating a new ticket.
         *
         * Uses Y.Form() to build and render the form dynamically and
         * re-implements its submission to POST JSON data (instead of
         * 'urlencoded'). Opens the just-created ticket on success.
         *
         * @method setupCreateForm
         */
        setupCreateForm: function (container) {
            var form_message = Y.Node.create('<div>');
            Y.one(container).appendChild(form_message);

            // Build and render a form for creating a new Ticket.
            var f = new Y.Form({
                boundingBox: container,
                action : '/api/v1/ticket/',
                method : 'post',
                children : [
                    {name: 'title', required: true, label: 'Title:'},
                    {name: 'description', label: 'Description:',
                     type: 'TextareaField'},
                    {name: 'owner', required: true, label: 'Owner:',
                     validator: Y.FormField.VALIDATE_EMAIL_ADDRESS},
                    {name: 'series', required: true, label: 'Series:',
                     type: 'WebUISelectField', useDefaultOption: false,
                     choices: this.seriesChoices},
                    {name: 'sync_request', label: 'Sync request:'},
                    {name: 'sources', label: 'Source Packages:',
                     type: 'TextareaField'},
                    {name: 'landers', label: 'Landers:'},
                    {name: 'test_notes', label: 'Test notes:',
                     type: 'TextareaField'},
                    {name: 'comments', label: 'Comments:',
                     type: 'TextareaField'},
                    {name: 'create-button', type: 'SubmitButton',
                     value: 'Create!'}
                ]
            });
            f.render();

            // Hijack the default (www-urlencoded) form submission and
            // submit its content as JSON.
            f.getField('create-button').on('click', function(e) {
                e.halt();
                if (!f._runValidation()) {
                    return 1;
                }

                // XXX cprov 20140922: extend the form data with appropriate
                // 'status' and 'current_workflow_step'.
                var form_data = f.toJSON();

                // Convert form 'sources' to 'subtickets'.
                var sources_list = form_data['sources'].split('\n');
                delete form_data['sources'];
                form_data['subtickets'] = [];
                sources_list.forEach( function(name) {
                    form_data['subtickets'].push({
                        'source_package_upload': {
                            'sourcepackage': {'name': name},
                            'version': '1.0',
                            'architecture': 'all'},
                        'assignee': form_data['owner']
                    });
                });

                // Convert several form fields to 'citrain_overlay'.
                var citrain_overlay_fields = [
                    'comments', 'test_notes', 'landers', 'sync_request',
                    'job_url'
                ]
                var citrain_overlay_obj = {}
                citrain_overlay_fields.forEach( function(name) {
                    citrain_overlay_obj[name] = form_data[name];
                    delete form_data[name];
                });
                form_data['citrain_overlay'] = citrain_overlay_obj;

                Y.io(f.get('action'), {
                    method: f.get('method'),
                    sync: true,
                    data: Y.JSON.stringify(form_data),
                    headers: {
                        'Content-Type': 'application/json;',
                    },
                    on: {
                        success: function(tx, r) {
                            var location = r.getResponseHeader('Location');
                            Y.config.win.location.href = location.replace(
                                '/api/v1', '');
                        },
                        failure: function (tx, r) {
                            Y.log('FAILURE: ' + r.responseText);
                            form_message.addClass('form-error')
                            form_message.set(
                                'text', 'Ticket creation failed!')
                        }
                    }
                });
            });
        }
    };
}, '0.0.1', {requires: ['io', 'node', 'json-parse', 'json-stringify',
                        'gallery-form']});
